@@ -1,5 +1,5 @@
 
-MCGHD <- function(data=NULL, gpar0=NULL, G=2, max.iter=100, epsilon=1e-2,  label=NULL ) {
+MCGHD <- function(data=NULL, gpar0=NULL, G=2, max.iter=100, epsilon=1e-2,  label=NULL, method="kmeans" ) {
 	data=as.matrix(data)
 	data=scale(data)
 	if (is.null(data)) stop('data is null')
@@ -8,15 +8,17 @@ MCGHD <- function(data=NULL, gpar0=NULL, G=2, max.iter=100, epsilon=1e-2,  label
 	if (any(is.na(data))) stop('No NAs allowed.')
 	if (is.null(G)) stop('G is NULL')
 	if ( G < 1) stop('G is not a positive integer')
-	if (  max.iter< 1) stop('n is not a positive integer')
+	if (  max.iter< 1) stop('max.iter is not a positive integer')
 	
 	
-	if (is.null(gpar0)) gpar  = rmgpar(g=G,p=ncol(data),data=data)
+	if (is.null(gpar0)) gpar  = rmgpar(g=G,p=ncol(data),data=data, method=method)
 	else gpar = gpar0
 	
+    
+    
 	loglik = numeric(max.iter)
 	for (i in 1:3) {
-		gpar = EMgrstep(data=data, gpar=gpar, v=1, label = label)	
+		gpar = EMgrstep(data=data, gpar=gpar, v=1, label = label)
 		loglik[i] = llik(data, gpar)
 	}
 	
@@ -27,8 +29,9 @@ MCGHD <- function(data=NULL, gpar0=NULL, G=2, max.iter=100, epsilon=1e-2,  label
 		
 	}
 	pcol=ncol(data)
-	BIC=2*loglik[max.iter]-log(nrow(data))*(2*(G-1)+G*(2*pcol+0.5*pcol*(pcol-1))+G*2*pcol+G*2)
-#BIC=2*loglik[max.iter]-log(nrow(data))*((G-1)+G*(3*pcol+0.5*pcol*(pcol-1)))
+	#BIC=2*loglik[max.iter]-log(nrow(data))*(2*(G-1)+G*(2*pcol+0.5*pcol*(pcol-1))+G*2*pcol+G*2)
+    BIC=2*loglik[max.iter]-log(nrow(data))*((G-1)+G*(4*pcol+2+pcol*(pcol-1)/2))
+
 	par=partrue(gpar,G)
 	val = list(loglik= loglik[1:i], gpar=gpar,par=par, z=weights(data=data, gpar= gpar), map=MAP(data=data, gpar= gpar, label=label),BIC=BIC )
 	return(val)
