@@ -1,5 +1,5 @@
 
-MGHFA<- function(data=NULL, gpar0=NULL, G=2, n=100, label =NULL  ,q=2,epsilon=1e-2 ) {
+MGHFA<- function(data=NULL, gpar0=NULL, G=2, n=100, label =NULL  ,q=2,epsilon=1e-2, method="kmeans" ) {
 ##Expexctation Maximization estimation of GHD
 ##data
 ## G n clusters
@@ -14,7 +14,7 @@ MGHFA<- function(data=NULL, gpar0=NULL, G=2, n=100, label =NULL  ,q=2,epsilon=1e
 	if ( q < 1) stop('n is not a positive integer')	
 	
 	data=scale(data)
-	if (is.null(gpar0)) gpar = igparM(data=data, g=G,q=q)
+	if (is.null(gpar0)) gpar = igparM(data=data, g=G,q=q,method=method)
 	else gpar  = gpar0
 	loglik = numeric(n)
 	for (i in 1:3) {
@@ -27,7 +27,7 @@ MGHFA<- function(data=NULL, gpar0=NULL, G=2, n=100, label =NULL  ,q=2,epsilon=1e
 		loglik[i] = llikFA(data, gpar) ##likelyhood
 	}
 	pcol=ncol(data)
-	BIC=2*loglik[n]-log(nrow(data))*((G-1)+G*(3*pcol+2+pcol*q))
+	BIC=2*loglik[n]-log(nrow(data))*((G-1)+G*(3*pcol+2+pcol*q-q*(q-1)/2))
 	val = list(loglik= loglik, gpar=gpar, z=weightsFA(data=data, gpar= gpar), map=MAPFA(data=data, gpar= gpar, label=label) , BIC=BIC)
 	return(val)
 }
